"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const webpack_1 = require("webpack");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path = __importStar(require("path"));
const constants_1 = require("./constants");
const express_1 = __importDefault(require("express"));
const kolorist_1 = require("kolorist");
const DEFAULT_PLUGIN_OPTIONS = {
    disable: false,
    exclude: /node_modules/,
};
class InspectWebpackPlugin {
    options = DEFAULT_PLUGIN_OPTIONS;
    transformMap = {};
    context = '';
    done = false;
    constructor(options = {}) {
        this.options = {
            ...this.options,
            ...options,
        };
    }
    apply(compiler) {
        if (this.options.disable) {
            return;
        }
        this.context = compiler.context;
        compiler.hooks.done.tap(constants_1.PLUGIN_NAME, () => {
            this.done = true;
            this.createServer();
        });
        compiler.hooks.compilation.tap(constants_1.PLUGIN_NAME, (compilation) => {
            const normalModuleLoader = webpack_1.NormalModule.getCompilationHooks(compilation).loader;
            normalModuleLoader.tap(constants_1.PLUGIN_NAME, (loaderContext) => {
                loaderContext[constants_1.PLUGIN_NAME] = this.recordTransformInfo;
            });
            compilation.hooks.buildModule.tap(constants_1.PLUGIN_NAME, (module) => {
                // @ts-expect-error `resource` doesn't exist in `Module` type, but it exists exactly.
                const { resource: id } = module;
                if (id && !this.options.exclude?.test(id)) {
                    // @ts-expect-error `loaders` doesn't exist in `Module` type, but it exists exactly.
                    module.loaders.unshift(require.resolve('./loader'));
                    if (!this.transformMap[id]) {
                        // @ts-expect-error `resourceResolveData` doesn't exist in `Module` type, but it exists exactly.
                        const code = fs_extra_1.default.readFileSync(module.resourceResolveData.path, 'utf-8');
                        const start = Date.now();
                        this.transformMap[id] = [
                            {
                                name: '__LOAD__',
                                result: code,
                                start,
                                end: start,
                            },
                        ];
                    }
                }
            });
        });
    }
    recordTransformInfo = (result) => {
        if (!result) {
            return;
        }
        const { id, ...transformInfo } = result;
        // Do not append the transform map if it existed.
        if (this.transformMap[id] && this.transformMap[id].slice(-1)[0]?.name === transformInfo.name) {
            return;
        }
        if (this.transformMap[id]) {
            this.transformMap[id].push({
                ...transformInfo,
            });
        }
    };
    createServer() {
        const app = (0, express_1.default)();
        app.use(express_1.default.static(path.join(__dirname, '../client')));
        app.get('/data', (req, res) => {
            res.setHeader('Access-Control-Allow-Origin', '*');
            res.send({
                done: this.done,
                transformMap: this.transformMap,
                context: this.context,
            });
        });
        app.listen(constants_1.PORT, () => {
            console.log('\n');
            console.log((0, kolorist_1.green)((0, kolorist_1.bold)('Inspect is running at: ')) + (0, kolorist_1.blue)((0, kolorist_1.bold)(constants_1.SERVER_HOST)));
            console.log('\n');
        });
    }
}
exports.default = InspectWebpackPlugin;
//# sourceMappingURL=index.js.map